<?php

class actionGradesVote extends cmsAction{

    public function run(){

        if (!$this->request->isAjax()){ cmsCore::error404(); }
		
        // включено ли голосование от гостей
		$options = $this->getOptions();
        if(empty($options['allow_guest_vote']) && !$this->cms_user->is_logged){
            return $this->cms_template->renderJSON(array(
                'success' => false,
                'message' => LANG_ERROR
            ));
        }
        // Получаем параметры
        $direction         = $this->request->get('direction', '');
        $target_controller = $this->request->get('controller', '');
        $target_subject    = $this->request->get('subject', '');
        $target_id         = $this->request->get('id', 0);

        $template = cmsTemplate::getInstance();

        $is_valid = ($this->validate_sysname($target_controller)===true) &&
                    ($this->validate_sysname($target_subject)===true) &&
                    is_numeric($target_id) &&
                    is_numeric($direction);	
		
        if (!$is_valid){ $template->renderJSON(array('success' => false)); }

        $user = cmsUser::getInstance();

        // Объединяем всю информацию о голосе
        $vote = array(
            'user_id' => $user->id,
            'target_controller' => $target_controller,
            'target_subject' => $target_subject,
            'target_id' => $target_id,
            'score' => $direction
        );

        // Этот голос уже учитывался?
        $is_voted = $this->model->isUserVoted($vote);
        if ($is_voted){ $template->renderJSON(array('success' => false)); }
		
        $target_model = cmsCore::getModel('content');			
        $target = $this->model->getGradesTarget($target_model, $target_subject, $target_id);

        if (!empty($target['user_id'])){
            if ($target['user_id'] == $user->id){
                $template->renderJSON(array('success' => false));
            }
        }

        // Добавляем голос в лог
        $this->model->addVote($vote);

        // Обновляем суммарный рейтинг цели
        $total = $this->model->getCurrentVotesCount($target_subject, $target_id);
        $votes = $this->model->getCurrentVotes($target_subject, $target_id);
        if ($votes){
            $summ_votes = 0;
            foreach ($votes as $value) {
                $summ_votes = (int)$summ_votes + $value['score'];
            } 
        }
        $grades = round((int)$summ_votes/$total, 0);
        $this->model->updateGrades($target_model, $target_subject, $target_id, $grades);

        // Оповещаем всех об изменении рейтинга
        cmsEventsManager::hook('grades_vote', array(
            'subject' => $target_subject,
            'id' => $target_id,
            'target' => $target,
            'vote' => $vote,
            'grades' => $grades
        ));

        // Собираем результат
        $result = array(
            'success' => true,
            'grades' => $grades,
            'css_class' => html_signed_class( $grades ) . ($this->options['is_show'] ? ' clickable' : ''),
            'message' => LANG_GRADES_VOTED
        );

        $template->renderJSON($result);

    }

}
